"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.fileLoader = void 0;
const path_1 = require("path");
const debug_util_1 = require("../utils/debug.util");
const load_package_util_1 = require("../utils/load-package.util");
let parseToml;
let cosmiconfig;
const loadToml = function loadToml(filepath, content) {
    parseToml = (0, load_package_util_1.loadPackage)('@iarna/toml', "fileLoader's ability to parse TOML files").parse;
    try {
        const result = parseToml(content);
        return result;
    }
    catch (error) {
        error.message = `TOML Error in ${filepath}:\n${error.message}`;
        throw error;
    }
};
const getSearchOptions = (options) => {
    if (options.absolutePath) {
        return {
            searchPlaces: [(0, path_1.basename)(options.absolutePath)],
            searchFrom: (0, path_1.dirname)(options.absolutePath),
        };
    }
    const { basename: name = '.env', loaders = {} } = options;
    const additionalFormats = Object.keys(loaders).map(ext => ext.slice(1));
    const formats = [...additionalFormats, 'toml', 'yaml', 'yml', 'json', 'js'];
    return {
        searchPlaces: [
            ...formats.map(format => `${name}.${process.env.NODE_ENV}.${format}`),
            ...formats.map(format => `${name}.${format}`),
        ],
        searchFrom: options.searchFrom,
    };
};
/**
 * Will fill in some placeholders.
 * @param template - Text with placeholders for `data` properties.
 * @param data - Data to interpolate into `template`.
 *
 * @example
 ```
 placeholderResolver('Hello ${name}', {
    name: 'John',
  });
 //=> 'Hello John'
 ```
 */
const placeholderResolver = (template, data, disallowUndefinedEnvironmentVariables) => {
    const replace = (placeholder, key) => {
        let value = data;
        for (const property of key.split('.')) {
            value = value[property];
        }
        if (!value && disallowUndefinedEnvironmentVariables) {
            throw new Error(`Environment variable is not set for variable name: '${key}'`);
        }
        return String(value);
    };
    // The regex tries to match either a number inside `${{ }}` or a valid JS identifier or key path.
    const braceRegex = /\${(\d+|[a-z$_][\w\-$]*?(?:\.[\w\-$]*?)*?)}/gi;
    return template.replace(braceRegex, replace);
};
/**
 * File loader loads configuration with `cosmiconfig` from file system.
 *
 * It is designed to be easy to use by default:
 *  1. Searching for configuration file starts at `process.cwd()`, and continues
 *     to search up the directory tree until it finds some acceptable configuration.
 *  2. Various extensions are supported, such as `.json`, `.yaml`, `.toml`, `.js` and `.cjs`.
 *  3. Configuration base name defaults to .env (so the full name is `.env.json` or `.env.yaml`),
 *     separate file for each environment is also supported. For example, if current `NODE_ENV` is
 *     development, `.env.development.json` has higher priority over `.env.json`.
 *
 * @see https://github.com/davidtheclark/cosmiconfig
 * @param options cosmiconfig initialize options. See: https://github.com/davidtheclark/cosmiconfig#cosmiconfigoptions
 */
const fileLoader = (options = {}) => {
    cosmiconfig = (0, load_package_util_1.loadPackage)('cosmiconfig', 'fileLoader');
    const { cosmiconfigSync } = cosmiconfig;
    return () => {
        var _a, _b;
        const { searchPlaces, searchFrom } = getSearchOptions(options);
        const loaders = Object.assign({ '.toml': loadToml }, options.loaders);
        const explorer = cosmiconfigSync('env', Object.assign(Object.assign({ searchPlaces }, options), { loaders }));
        const result = explorer.search(searchFrom);
        if (!result) {
            throw new Error(`Failed to find configuration file.`);
        }
        (0, debug_util_1.debug)(`File-loader has loaded a configuration file from ${result.filepath}`);
        let config = result.config;
        if (!((_a = options.ignoreEnvironmentVariableSubstitution) !== null && _a !== void 0 ? _a : true)) {
            const replacedConfig = placeholderResolver(JSON.stringify(result.config), process.env, (_b = options.disallowUndefinedEnvironmentVariables) !== null && _b !== void 0 ? _b : true);
            config = JSON.parse(replacedConfig);
        }
        return config;
    };
};
exports.fileLoader = fileLoader;
//# sourceMappingURL=file-loader.js.map